package com.ic;

import java.util.TimeZone;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.web.servlet.support.SpringBootServletInitializer;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.context.annotation.Bean;
import org.springframework.data.domain.AuditorAware;
import org.springframework.data.jpa.convert.threeten.Jsr310JpaConverters;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.scheduling.annotation.EnableScheduling;

import com.ic.audit.SpringSecurityAuditorAware;
import com.ic.constant.DBConfig;
import com.ic.constant.TemplateConfig;

//@EnableAutoConfiguration(exclude = { DataSourceAutoConfiguration.class, MultipartAutoConfiguration.class })
@SpringBootApplication
@EnableCaching
@EnableJpaAuditing(auditorAwareRef = "auditorAware")
@EntityScan(basePackageClasses = { 
		IncomeCalcApplication.class,
		Jsr310JpaConverters.class 
})
public class IncomeCalcApplication extends SpringBootServletInitializer {

	
	@PostConstruct
	void init() {
		TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
	}
	
	@Autowired 
	DBConfig DBConfig;
	
	@Autowired
	TemplateConfig TemplateConfig;
	
	
	@Bean 
    public AuditorAware<String> auditorAware() { 
        return new SpringSecurityAuditorAware();
    }

	/*@Override
    protected SpringApplicationBuilder configure(SpringApplicationBuilder application) {
        return application.sources(IncomeCalcApplication.class);
    }*/
	
	@Override
    protected SpringApplicationBuilder configure(SpringApplicationBuilder application) {
        return application.sources(IncomeCalcApplication.class);
    }

	public static void main(String[] args) {
	
		SpringApplication.run(IncomeCalcApplication.class, args);
		
	}
	

}
 