package com.ic.aspect;

import java.io.IOException;
import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.fasterxml.jackson.core.JsonEncoding;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ic.annotation.AuditLogs;
import com.ic.constant.StatusCode;
import com.ic.modules.auth.payload.AuditLogsDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.payload.request.LoginRequest;
import com.ic.modules.auth.service.AuditLogsService;
import com.ic.security.UserPrincipal;

@Aspect
@Component
public class ServiceVersionAspect {

	@Autowired
	AuditLogsService auditLogsService;
	
	@Around("@annotation(com.ic.annotation.AuditLogs)")
	public ResponseEntity<?> auditLog(ProceedingJoinPoint joinPoint) throws Throwable {
		
		long startTime = System.currentTimeMillis();
		
		
	    ResponseEntity re = null;
	    if(joinPoint.getArgs().length > 0) {
	    	ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
		    Validator validator = validatorFactory.usingContext().getValidator();
		    Set<ConstraintViolation<Object>> constrains = validator.validate(joinPoint.getArgs()[0]);
		    if(constrains.size() > 0) {
		    	String message = "";
			    for (ConstraintViolation constrain : constrains) {
			    	message += "[" + constrain.getPropertyPath() + ":" + constrain.getMessage() + "]";
			    }
			    re = new ResponseEntity(new ApiResponse(StatusCode.INVALID_ARGUMENT, true, message, null),HttpStatus.BAD_REQUEST);
		    }else {
		    	re = (ResponseEntity)joinPoint.proceed();
		    }
	    }else {
	    	re = (ResponseEntity)joinPoint.proceed();
	    }
		
        long timeTaken = System.currentTimeMillis() - startTime;
        ApiResponse ar = (ApiResponse)re.getBody();
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest();
		String remoteAddr = "";
        if (request != null) {
            remoteAddr = request.getHeader("X-FORWARDED-FOR");
            if (remoteAddr == null || "".equals(remoteAddr)) {
                remoteAddr = request.getRemoteAddr();
            }
        }
       
		UserPrincipal userPrincipal;
		if(SecurityContextHolder.getContext().getAuthentication().getPrincipal() instanceof String) {
			userPrincipal = new UserPrincipal();
			try {
				LoginRequest loginRequest = (LoginRequest)joinPoint.getArgs()[0];
				
				userPrincipal.setBrowserInfo(loginRequest.getBrowserInfo());
				userPrincipal.setDeviceInfo(loginRequest.getDeviceInfo());
				userPrincipal.setOS(loginRequest.getOS());
				userPrincipal.setUsername(loginRequest.getUsername());
			}catch(Exception e) {
				
				userPrincipal.setBrowserInfo(getClientBrowser(request));
				userPrincipal.setDeviceInfo(null);
				userPrincipal.setOS(getClientOS(request));
				userPrincipal.setUsername("NA");
			}
		}else {
			userPrincipal = (UserPrincipal)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		}
		
		String header = request.getHeader("Authorization");

		Method method = MethodSignature.class.cast(joinPoint.getSignature()).getMethod();
		if(method.isAnnotationPresent(AuditLogs.class)) {
			AuditLogs name = method.getAnnotation(AuditLogs.class);
		    Timestamp timestamp = new Timestamp(System.currentTimeMillis());
		    
		    com.ic.modules.auth.model.AuditLogs auditLogs = new com.ic.modules.auth.model.AuditLogs(null, name.service(), remoteAddr,
		    		userPrincipal.getDeviceInfo(), userPrincipal.getBrowserInfo(), userPrincipal.getUsername(), userPrincipal.getOS(), 
		    		ar.getCode(), ar.getMessage(), timeTaken, joinPoint.getArgs().length>0?joinPoint.getArgs()[0].toString():null , ar.getMessage(),new Date());
		    
		    AuditLogsDto auditLogsDto = new AuditLogsDto(auditLogs);
		    if(!name.isRequest()) {
		    	auditLogsDto.setRequestObj(null);
		    }
		    if(!name.isResponse()) {
		    	auditLogsDto.setResponseObj(null);
		    }
		    auditLogsService.save(auditLogsDto);
//		    if(name.isResponse()==false) {
//				return null;
//			}
//			else {
//				return re;
//			}
		}
        //Annotation[] parametersAnnotations = method.getAnnotations();
		
	    return re;
	}
	private String getValue(Object result) {
		String returnValue = null;
		if (null != result) {
			if (result.toString().endsWith("@" + Integer.toHexString(result.hashCode()))) {
				returnValue = ReflectionToStringBuilder.toString(result);
			} else {
				returnValue = result.toString();
			}
		}
		return returnValue;
	}
	
private ObjectMapper beanToJSON(Object bean){
	 JsonGenerator jsonGenerator = null;
	 ObjectMapper objectMapper = null;
  objectMapper = new ObjectMapper();
  try{
     jsonGenerator = objectMapper.getJsonFactory().createJsonGenerator(System.out, JsonEncoding.UTF8);
     jsonGenerator.writeObject(bean); 
     objectMapper.writeValue(System.out, bean);
     jsonGenerator.flush();
     jsonGenerator.close();
  }catch (IOException e) {
        e.printStackTrace();
  }
  return objectMapper;
  
	}


public String getReferer(HttpServletRequest request) {
    final String referer = request.getHeader("referer");
    return referer;
}

public String getFullURL(HttpServletRequest request) {
    final StringBuffer requestURL = request.getRequestURL();
    final String queryString = request.getQueryString();

    final String result = queryString == null ? requestURL.toString() : requestURL.append('?')
            .append(queryString)
            .toString();

    return result;
}

//http://stackoverflow.com/a/18030465/1845894
public String getClientIpAddr(HttpServletRequest request) {
    String ip = request.getHeader("X-Forwarded-For");
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
        ip = request.getHeader("Proxy-Client-IP");
    }
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
        ip = request.getHeader("WL-Proxy-Client-IP");
    }
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
        ip = request.getHeader("HTTP_CLIENT_IP");
    }
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
        ip = request.getHeader("HTTP_X_FORWARDED_FOR");
    }
    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
        ip = request.getRemoteAddr();
    }
    return ip;
}

//http://stackoverflow.com/a/18030465/1845894
public String getClientOS(HttpServletRequest request) {
    final String browserDetails = request.getHeader("User-Agent");

    //=================OS=======================
    final String lowerCaseBrowser = browserDetails.toLowerCase();
    if (lowerCaseBrowser.contains("windows")) {
        return "Windows";
    } else if (lowerCaseBrowser.contains("mac")) {
        return "Mac";
    } else if (lowerCaseBrowser.contains("x11")) {
        return "Unix";
    } else if (lowerCaseBrowser.contains("android")) {
        return "Android";
    } else if (lowerCaseBrowser.contains("iphone")) {
        return "IPhone";
    } else {
        return "UnKnown, More-Info: " + browserDetails;
    }
}

//http://stackoverflow.com/a/18030465/1845894
public String getClientBrowser(HttpServletRequest request) {
    final String browserDetails = request.getHeader("User-Agent");
    final String user = browserDetails.toLowerCase();

    String browser = "";

    //===============Browser===========================
    if (user.contains("msie")) {
        String substring = browserDetails.substring(browserDetails.indexOf("MSIE")).split(";")[0];
        browser = substring.split(" ")[0].replace("MSIE", "IE") + "-" + substring.split(" ")[1];
    } else if (user.contains("safari") && user.contains("version")) {
        browser = (browserDetails.substring(browserDetails.indexOf("Safari")).split(" ")[0]).split(
                "/")[0] + "-" + (browserDetails.substring(
                browserDetails.indexOf("Version")).split(" ")[0]).split("/")[1];
    } else if (user.contains("opr") || user.contains("opera")) {
        if (user.contains("opera"))
            browser = (browserDetails.substring(browserDetails.indexOf("Opera")).split(" ")[0]).split(
                    "/")[0] + "-" + (browserDetails.substring(
                    browserDetails.indexOf("Version")).split(" ")[0]).split("/")[1];
        else if (user.contains("opr"))
            browser = ((browserDetails.substring(browserDetails.indexOf("OPR")).split(" ")[0]).replace("/",
                                                                                                       "-")).replace(
                    "OPR", "Opera");
    } else if (user.contains("chrome")) {
        browser = (browserDetails.substring(browserDetails.indexOf("Chrome")).split(" ")[0]).replace("/", "-");
    } else if ((user.indexOf("mozilla/7.0") > -1) || (user.indexOf("netscape6") != -1) || (user.indexOf(
            "mozilla/4.7") != -1) || (user.indexOf("mozilla/4.78") != -1) || (user.indexOf(
            "mozilla/4.08") != -1) || (user.indexOf("mozilla/3") != -1)) {
        //browser=(userAgent.substring(userAgent.indexOf("MSIE")).split(" ")[0]).replace("/", "-");
        browser = "Netscape-?";

    } else if (user.contains("firefox")) {
        browser = (browserDetails.substring(browserDetails.indexOf("Firefox")).split(" ")[0]).replace("/", "-");
    } else if (user.contains("rv")) {
        browser = "IE";
    } else {
        browser = "UnKnown, More-Info: " + browserDetails;
    }

    return browser;
}

public String getUserAgent(HttpServletRequest request) {
    return request.getHeader("User-Agent");
}
}

