package com.ic.config;

import java.util.HashMap;

import javax.annotation.Resource;
import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;

import org.hibernate.cfg.AvailableSettings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

@Configuration
@PropertySource("classpath:application.properties")
@EnableTransactionManagement
@EnableJpaRepositories(
	entityManagerFactoryRef = "iRecruitSchemaEntityManagerFactory", 
	transactionManagerRef = "iRecruitSchemaTransactionManager", 
	basePackages = {
		"com.ic.modules.auth.repository"
	})
//@Resource(name="jdbc/", type=javax.sql.DataSource.class, lookup="jdbc/fileUploadDS")
public class IRecruitSchemaConfig {

	 @Autowired
	 Environment environment;
	 
	 //@Value("${irecruit.datasource.jndi-name}")
	 //private static String JNDI_NAME;
	 
	 private static final String JNDI_NAME = "jdbc/irecruit";
	@Primary
	@Bean(name = "iRecruitSchemaDataSource")
	@ConfigurationProperties(prefix = "irecruit.datasource")
	@Resource(name = JNDI_NAME)
	public DataSource iRecruitSchemaDataSource() {
		//HikariDataSource dataSource = new HikariDataSource();
		
		//dataSource.setDataSourceJNDI(environment.getProperty("irecruit.datasource.jndi-name"));
		/*dataSource.setDriverClassName(environment.getProperty("irecruit.datasource.driver-class-name"));
        dataSource.setJdbcUrl(environment.getProperty("irecruit.datasource.url"));
        dataSource.setUsername(environment.getProperty("irecruit.datasource.username"));
        dataSource.setPassword(environment.getProperty("irecruit.datasource.password"));
        dataSource.setCatalog("irecruitCat");
        */
        /*dataSource.setConnectionTimeout(Long.parseLong(environment.getProperty("spring.datasource.hikari.connectionTimeout")));
        dataSource.setValidationTimeout(Long.parseLong(environment.getProperty("spring.datasource.hikari.validationTimeout")));
        dataSource.setMaximumPoolSize(Integer.parseInt(environment.getProperty("spring.datasource.hikari.maximumPoolSize")));
        dataSource.setMaxLifetime(Long.parseLong(environment.getProperty("spring.datasource.hikari.maxLifetime")));
        dataSource.setIdleTimeout(Long.parseLong(environment.getProperty("spring.datasource.hikari.idleTimeout")));
        dataSource.setMinimumIdle(Integer.parseInt(environment.getProperty("spring.datasource.hikari.minimumIdle")));

        return dataSource;*/
		

	        JndiDataSourceLookup jndiDataSourceLookup = new JndiDataSourceLookup();
	        jndiDataSourceLookup.setResourceRef(true);
	        return jndiDataSourceLookup.getDataSource(JNDI_NAME);

	    
		//return DataSourceBuilder.create().build();
	}

	@Primary
	@Bean(name = "iRecruitSchemaEntityManagerFactory")
    public LocalContainerEntityManagerFactoryBean entityManagerFactory() {
        LocalContainerEntityManagerFactoryBean em = new LocalContainerEntityManagerFactoryBean();
        em.setDataSource(iRecruitSchemaDataSource());
        em.setPersistenceUnitName("iRecruitSchema");
        em.setPackagesToScan(new String[] { "com.ic.modules.auth.model"
		});
        HibernateJpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter();
        em.setJpaVendorAdapter(vendorAdapter);
        HashMap<String, Object> properties = new HashMap<>();
        properties.put(AvailableSettings.HBM2DDL_AUTO, environment.getProperty("irecruit.jpa.hibernate.ddl-auto"));
        properties.put(AvailableSettings.GLOBALLY_QUOTED_IDENTIFIERS, environment.getProperty("irecruit.jpa.hibernate.globally_quoted_identifiers"));
        properties.put(AvailableSettings.PHYSICAL_NAMING_STRATEGY, CustomPhysicalNamingStrategy.class.getName());
        em.setJpaPropertyMap(properties);
        return em;
    }
	
	@Primary
	@Bean(name = "iRecruitSchemaTransactionManager")
	public PlatformTransactionManager iRecruitSchemaTransactionManager(
			@Qualifier("iRecruitSchemaEntityManagerFactory") EntityManagerFactory iRecruitSchemaEntityManagerFactory) {
		
		JpaTransactionManager transactionManager  = new JpaTransactionManager();
		transactionManager.setEntityManagerFactory(entityManagerFactory().getObject());
		return transactionManager;
		//return new JpaTransactionManager(iRecruitSchemaEntityManagerFactory);
	}
}