package com.ic.config;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.springframework.data.jpa.domain.Specification;

import com.ic.modules.auth.payload.request.Condition;

public class QuerySpecification {
	  
	public static Specification<?> generateQuery(List<Condition> conditions) {
		return (root, query, cb) ->{ 
			
			List<Predicate> predicates = getPredicate(conditions,root,query,cb);
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
			
		};
	}
	public static List<Condition> getConditionByKey(List<Condition> conditions,String key ) {
		List<Condition> conditionArr = new ArrayList<Condition>();
		for(int cnt=0;cnt<conditions.size();cnt++) {
			Condition condition = conditions.get(cnt);
			if(condition.getIsGroup()) {
				conditionArr.addAll(getConditionByKey(condition.getGroupConditions(),key));
			}else {
				if(condition.getKey().equalsIgnoreCase(key)) {
					conditionArr.add(condition);
				}
			}
			
		}
		return conditionArr;
	}
	public static List<Predicate> getPredicate(List<Condition> conditions,Root<?> root, CriteriaQuery<?> query, CriteriaBuilder criteriaBuilder){
		List<Predicate> predicates = new ArrayList<>();
		for(int cnt=0;cnt<conditions.size();cnt++) {
			Condition condition = conditions.get(cnt);
			if(condition.getIsGroup()) {
				List<Predicate> predicatesGroup = getPredicate(condition.getGroupConditions(),root,query,criteriaBuilder);
				if(condition.getGroupCondition().equalsIgnoreCase("AND")) {
					predicates.add(criteriaBuilder.and(predicatesGroup.toArray(new Predicate[predicatesGroup.size()])));
				}else {
					predicates.add(criteriaBuilder.or(predicatesGroup.toArray(new Predicate[predicatesGroup.size()])));
				}
			}else {
				if(condition.getCriteria().equalsIgnoreCase("like")) {
					predicates.add(criteriaBuilder.like(root.get(condition.getKey()), "%" + (String)condition.getValue() + "%"));
				}else if(condition.getCriteria().equalsIgnoreCase("=")) {
					predicates.add(criteriaBuilder.equal(root.get(condition.getKey()),condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase(">")) {
					predicates.add(criteriaBuilder.gt(root.get(condition.getKey()),(Number)condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase(">=")) {
					predicates.add(criteriaBuilder.ge(root.get(condition.getKey()),(Number)condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase("<")) {
					predicates.add(criteriaBuilder.lt(root.get(condition.getKey()),(Number)condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase("<=")) {
					predicates.add(criteriaBuilder.le(root.get(condition.getKey()),(Number)condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase("in")) {
					predicates.add(root.get(condition.getKey()).in(condition.getValue()));
				}else if(condition.getCriteria().equalsIgnoreCase("notin")) {
					predicates.add(root.get(condition.getKey()).in(condition.getValue()).not());
				}else if(condition.getCriteria().equalsIgnoreCase("isNull")) {
					predicates.add(root.get(condition.getKey()).isNull());
				}else if(condition.getCriteria().equalsIgnoreCase("isNotNull")) {
					predicates.add(root.get(condition.getKey()).isNotNull());
				}
				
			}
		}
		return predicates;
	}
}