package com.ic.modules.auth.controller;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ic.annotation.AuditLogs;
import com.ic.constant.StatusCode;
import com.ic.exception.ResourceNotFoundException;
import com.ic.modules.auth.model.User;
import com.ic.modules.auth.payload.P100LoginResponse;
import com.ic.modules.auth.payload.UserDto;
import com.ic.modules.auth.payload.UserOTPDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.payload.request.JwtAuthenticationResponse;
import com.ic.modules.auth.payload.request.LoginRequest;
import com.ic.modules.auth.payload.request.OTPRequest;
import com.ic.modules.auth.payload.request.OTPValidateRequest;
import com.ic.modules.auth.payload.request.P100LoginRequest;

import com.ic.modules.auth.service.UserService;
import com.ic.security.AuthProvider;
import com.ic.security.JwtTokenProvider;
import com.ic.security.UserPrincipal;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;


@RestController
@RequestMapping("/api/v1/auth")
@Api(description = "Set of endpoints for Creating, Retrieving, Updating and Deleting of Persons.")
public class AuthController {
	
	@Autowired
	AuthProvider authenticationManager;

	@Autowired
	UserService userService;


	

	@Autowired
	JwtTokenProvider tokenProvider;
	



	private static final Logger LOG = LoggerFactory.getLogger(AuthController.class);

	@PostMapping("/signin")
	@ApiOperation("Sign-in user by userid and password")
	@AuditLogs(service="login",isResponse=true)
	public ResponseEntity<ApiResponse> authenticateUser (@RequestBody LoginRequest loginRequest, 
		    HttpServletRequest request, HttpServletResponse response) {
		
		
		//DataSourceContextHolder.setDataSourceName("iRecruitSchemaDataSource");
		Map<String, Object> returnUserObject = new HashMap<>();
		String jwt;
		
		if(loginRequest.getUsername().matches("[0-9]+")) {
            loginRequest.setUsername(String.format("%09d", Integer.parseInt(loginRequest.getUsername())));
        }
		
		try {
			
			UsernamePasswordAuthenticationToken authenticationObj = new UsernamePasswordAuthenticationToken(loginRequest.getUsername(), loginRequest.getUserId());
			authenticationObj.setDetails(loginRequest);
			Authentication authentication = authenticationManager.authenticate(authenticationObj);
			SecurityContextHolder.getContext().setAuthentication(authentication);

			
			//if (authentication.getAuthorities().stream().anyMatch(r -> r.getAuthority().equals(loginRequest.getRoleType()))) {
			jwt = tokenProvider.generateToken(authentication);
			UserPrincipal userPrincipal = (UserPrincipal) authentication.getPrincipal();
			User user = userService.findById(userPrincipal.getUsername());
			if(loginRequest.getNotificationId() != null) {
				user.setNotificationId(loginRequest.getNotificationId());
				userService.save(user);
			}
			returnUserObject.put("userDetail", new UserDto(user));
			returnUserObject.put("tokenDetail", new JwtAuthenticationResponse(jwt));
			LOG.info("Login Success"); 
			return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "LOGIN_SUCCESS", returnUserObject), HttpStatus.OK);
		} catch (ResourceNotFoundException e) {
			//LOG.error("Exception while Json processing: ", e);
			return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "Tenant not found", null), HttpStatus.UNAUTHORIZED);
		} catch (UsernameNotFoundException e) {
			//LOG.error("Exception while Json processing: ", e);
			return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "Username not found", null), HttpStatus.UNAUTHORIZED);
		}catch (BadCredentialsException e) {
			LOG.error("Wrong password: ");
			return new ResponseEntity(new ApiResponse(StatusCode.INVALID_CREDENTIAL, true, "Wrong username or password", null), HttpStatus.UNAUTHORIZED);
		}catch (JSONException e) {
			
			System.out.println(e);
			LOG.error("Exception while Json processing: ", e);
			return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "JSON_PROCESSING_EXCEPTION", null), HttpStatus.UNAUTHORIZED);
		}catch (Exception e) {
			
			System.out.println(e);
			LOG.error("Exception while Json processing: ", e);
			return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "JSON_PROCESSING_EXCEPTION", null), HttpStatus.UNAUTHORIZED);
		}

	}
	
	/*
	 * @PostMapping("/P100Login")
	 * 
	 * @ApiOperation("P100 Login")
	 * 
	 * @AuditLogs(isRequest = true, isResponse = true, service = "P100Login") public
	 * ResponseEntity<?> P100Login(@RequestBody P100LoginRequest p100LoginRequest) {
	 * try { P100LoginResponse p100LoginResponse =
	 * p100LoginService.p100Login(p100LoginRequest); return new ResponseEntity(new
	 * ApiResponse(HttpStatus.OK.value(), true, "P100 Login",
	 * p100LoginResponse),HttpStatus.OK); }catch (Exception e) { return new
	 * ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR,
	 * true,"Error in P100 Login", null), HttpStatus.UNAUTHORIZED); } }
	 */
	
	/*@PostMapping("/SendOTP")
	@ApiOperation("Send OTP")
	@AuditLogs(isRequest = true, isResponse = true, service = "SendOTP")
	public ResponseEntity<?> sendOTP(@RequestBody OTPRequest otpRequest) {
		try {
			UserOTPDto userOTPDto = userOTPService.create(otpRequest.getLeadId(),otpRequest.getMobileNo(),otpRequest.getEmailId(),otpRequest.getServiceName() );
			return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "P100 Login", userOTPDto),HttpStatus.OK);
		}catch (Exception e) {
			return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true,"Error while OTP sending", null), HttpStatus.UNAUTHORIZED);
		}
	}
	
	@PostMapping("/ValidateOTP")
	@ApiOperation("Validate OTP")
	@AuditLogs(isRequest = true, isResponse = true, service = "ValidateOTP")
	public ResponseEntity<?> validateOTP(@RequestBody OTPValidateRequest otpValidateRequest) {
		try {
			UserOTPDto userOTPDto = userOTPService.validateOTP(otpValidateRequest.getId(),otpValidateRequest.getLeadId(),otpValidateRequest.getOtp() );
			return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "P100 Login", userOTPDto),HttpStatus.OK);
		}catch (Exception e) {
			return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true,"Error while OTP sending", null), HttpStatus.UNAUTHORIZED);
		}
	}*/

}
