package com.ic.modules.auth.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.ic.annotation.AuditLogs;
import com.ic.constant.DBConfig;
import com.ic.constant.StatusCode;
import com.ic.exception.ResourceNotFoundException;
import com.ic.modules.auth.model.Configurations;
import com.ic.modules.auth.payload.ConfigurationsDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.service.ConfigurationsService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController 
@RequestMapping("/api/v1")
@Api(description = "Configurations management API")
public class ConfigurationsController {
  
    @Autowired
    ConfigurationsService configurationsService;
 
    @Autowired
    DBConfig DBConfig;
    @PostMapping("/configurations/get")
    @ApiOperation("Get all configurations list")
    @AuditLogs(isRequest = true, isResponse = true,service="Get Config")
    public ResponseEntity<?> getAllConfigurations( ){
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "Configurations List", configurationsService.findAll().stream().map(configurations -> new ConfigurationsDto(configurations))), HttpStatus.OK);
    }
 
    @PostMapping("/configurations/refresh")
    @ApiOperation("to Refresh all configurations list")
    @AuditLogs(isRequest = true, isResponse = true,service="Refresh Config")
    public ResponseEntity<?> refreshConfigurations( ){
    	DBConfig.refreshConfig();
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "Configurations has been refreshed", null), HttpStatus.OK);
    }
     
    @PostMapping("/configurations/create")
    @ApiOperation("Create configurations")
    @AuditLogs(isRequest = true, isResponse = true,service="Create Config")
    public ResponseEntity<?> createConfigurations(@RequestBody ConfigurationsDto configurations) {
    	
    	System.out.println("in configuration");
    	Configurations config = null;
    	try {
	    	config =  configurationsService.save(configurations,true);

    	}catch(ResourceNotFoundException e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "Configurations Not found", configurations.getKey()), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "Configurations creation failed", configurations.getKey()), HttpStatus.BAD_REQUEST);
    	}
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "Configurations added successfully", new ConfigurationsDto(config)), HttpStatus.OK);
    	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }

    @PostMapping("/configurations/update")
    @ApiOperation("Update configurations")
    @AuditLogs(isRequest = true, isResponse = true,service="Update Config")
    public ResponseEntity<?> updateConfigurations(@RequestBody ConfigurationsDto configurations) {
    	try {
    		Configurations newUSer =  configurationsService.save(configurations,false);
    		return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "Configurations updated successfully", new ConfigurationsDto(newUSer)), HttpStatus.OK);
    	}catch(DataIntegrityViolationException e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "Configurations data already exist", null), HttpStatus.ALREADY_REPORTED);
    	}catch(ResourceNotFoundException e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.INVALID_USER, true, "Configurations record not found", null), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "Configurations updation failed", null), HttpStatus.BAD_REQUEST);
    	}
    	//return new ResponseEntity(new ApiResponse(StatusCode.DEPENDANCY_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }

}
