package com.ic.modules.auth.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.ic.annotation.AuditLogs;
import com.ic.constant.LeaderCalcConfig;
import com.ic.constant.StatusCode;
import com.ic.exception.ResourceNotFoundException;
import com.ic.modules.auth.model.L1slabsMapping;
import com.ic.modules.auth.model.MpbMapping;
import com.ic.modules.auth.payload.L1slabsDto;
import com.ic.modules.auth.payload.mpbDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.service.L1slabsMappingService;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController 
@RequestMapping("/api/v1")
@Api(description = "L1 Slabs maping management API")

public class L1slabsMappingController {

	@Autowired
	  L1slabsMappingService l1slabsMappingService;
	
	LeaderCalcConfig LeaderCalcConfig;
    @PostMapping("/l1salbsmapping/get")
    @ApiOperation("Get all l1slabs mapping list")
    @AuditLogs(isRequest = true, isResponse = true,service="Get l1slabs")
  //Neel – 20-Apr-2020 – Tata - l1 slabs mapping – Added function for fetch all data of l1 slabs from table
    public ResponseEntity<?> getAllL1slabsMappings( ){
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l1 slabs List", l1slabsMappingService.findAll().stream().map(L1slabsMapping -> new L1slabsDto(L1slabsMapping))), HttpStatus.OK);
    }
    @PostMapping("/l1salbsmapping/create")
    @ApiOperation("Create l1salbsmapping")
    @AuditLogs(isRequest = true, isResponse = true,service="Create l1salbsmapping")
  //Neel – 20-Apr-2020 – Tata - l1 slabs mapping – Added function for store all data of l1 slabs  table
    public ResponseEntity<?> createL1slabsmapping(@RequestBody L1slabsDto l1slabsmapping) {
    	// System.out.println("in l1salbsmapping contrroler");
    	// System.out.println("in l1salbsmapping");
    	L1slabsMapping l1 = null;
    	try {
    		l1 =  l1slabsMappingService.save(l1slabsmapping,true);

    	}catch(ResourceNotFoundException e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "l1mapping Not found", l1.getId()), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "l1mapping creation failed", l1.getId()), HttpStatus.BAD_REQUEST);
    	}
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l1mapping added successfully", new L1slabsDto(l1)), HttpStatus.OK);
    	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }
    
    
    @PostMapping("/l1salbsmapping/update")
    @ApiOperation("update l1salbsmapping")
    @AuditLogs(isRequest = true, isResponse = true,service="Create l1salbsmapping")
    public ResponseEntity<?> updateL1slabsmapping(@RequestBody L1slabsDto l1slabsmapping) {
    	// System.out.println("in l1salbsmapping contrroler");
    	// System.out.println("in l1salbsmapping");
    	L1slabsMapping l1 = null;
    	try {
    		l1 =  l1slabsMappingService.save(l1slabsmapping,false);

    	}catch(ResourceNotFoundException e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "l1mapping Not found", l1.getId()), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		e.printStackTrace();
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "l1mapping creation failed", l1.getId()), HttpStatus.BAD_REQUEST);
    	}
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l1mapping added successfully", new L1slabsDto(l1)), HttpStatus.OK);
    	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }
    
}
