package com.ic.modules.auth.controller;

import java.util.HashMap;
import java.util.Map;

import javax.validation.Valid;

import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import org.json.JSONException;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.ic.annotation.AuditLogs;
import com.ic.constant.LeaderCalcConfig;
import com.ic.constant.StatusCode;
import com.ic.exception.ResourceNotFoundException;
import com.ic.modules.auth.model.L1slabsMapping;
import com.ic.modules.auth.model.MpbMapping;
import com.ic.modules.auth.payload.L1slabsDto;
import com.ic.modules.auth.payload.mpbDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.service.L1slabsMappingService;
import com.ic.modules.auth.service.LeaderCalcService;
import com.ic.modules.auth.service.NGPAtoNGPLCalcService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController 
@RequestMapping("/api/v1")
@Api(description = "NGPL Calc API")


public class NGPAtoNGPLCalc {
	@Autowired
	NGPAtoNGPLCalcService nGPAtoNGPLCalcService;
	LeaderCalcConfig LeaderCalcConfig;
	 @PostMapping(value = "/ngpatongplCalculation", headers = "Accept=application/json")
	    @ApiOperation("Create ngplcalc")
 @AuditLogs(isRequest = true, isResponse = true,service="ngpatongplcalc")
	 @JsonSerialize
	 public ResponseEntity<?> CalculateNGPL(@RequestBody Map<String, Object> lookupRequestObject) throws JsonProcessingException {
	    	// System.out.println("in ngpl contrroler");
	    	
	    	 ObjectMapper mapper = new ObjectMapper();
	    	 
	    	 String data = mapper.writeValueAsString(lookupRequestObject);
	    	 JSONObject requestJSON = new JSONObject(data);
	    	 HashMap<String,Object> result  = null;
	    
	    	JSONObject calc = null;
	    	try {
				// System.out.println("in try");
				// Yashvant - 16 May 2020 - 0000 - Validation For JSON Keys ( < , > )
				if( checkValidation(requestJSON) ) {
					calc =  nGPAtoNGPLCalcService.prepareAllFormula(requestJSON);
				}
				else {
					return new ResponseEntity( new ApiResponse( StatusCode.UNEXPECTED_ERROR, true, "Request Contains Invalid Input", calc ), HttpStatus.BAD_REQUEST );
				}
	    		// System.out.println("after operation calc");
	    		// System.out.println(calc);
	    		
	    		
	    		result =new HashMap<String, Object>();
	    		result.put("response", calc);
	    		// System.out.println(result);

	    	}catch(ResourceNotFoundException e) {
	    		e.printStackTrace();
	    	//	return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "fycmapping Not found", fyc.getId()), HttpStatus.NOT_FOUND);
	    	}catch(Exception e) {
	    		e.printStackTrace();
	    		//return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "fycmapping creation failed", fyc.getId()), HttpStatus.BAD_REQUEST);
	    	}
	    	
			
			return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "calculation done successfully", calc.toString()), HttpStatus.OK);
	    	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
	    }
	
		// Yashvant - 16 May 2020 - 0000 - Validation For JSON Keys ( < , > )
		public boolean checkValidation(JSONObject request) throws JSONException {
			// No need of try catch as default exception added
			int i;
			String [] jsonKeysStringValue = new String [6];
			
			jsonKeysStringValue[0] = request.getString("FS_Type");
			jsonKeysStringValue[1] = request.getString("Total_Validations_Met");

			request = request.getJSONObject("Metrics_MeetingConversion_Earnings");

			jsonKeysStringValue[2] = request.getJSONObject("Year_1_NGPA").getString("Validations_Met");
			jsonKeysStringValue[3] = request.getJSONObject("Year_1_NGPL").getString("Validations_Met");
			jsonKeysStringValue[4] = request.getJSONObject("Year_1_Total").getString("Validations_Met");
			jsonKeysStringValue[5] = request.getJSONObject("Year_2_4_Validated_NGPAs").getString("Validations_Met");
			
			for( i = 0; i < jsonKeysStringValue.length; i++ ) {
				if( jsonKeysStringValue[i].contains("<") && jsonKeysStringValue[i].contains(">") ) {
					return false;
				}
			}
			
			return true;
		}
}
