package com.ic.modules.auth.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.ic.annotation.AuditLogs;
import com.ic.constant.StatusCode;
import com.ic.exception.ResourceNotFoundException;
import com.ic.modules.auth.model.User;
import com.ic.modules.auth.payload.UserDto;
import com.ic.modules.auth.payload.request.ApiResponse;
import com.ic.modules.auth.service.UserService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

//@RestController 
@RequestMapping("/api/v1")
@Api(description = "User management API")
public class UserController {
  
    @Autowired
    UserService userService;
 

    @GetMapping("/user")
    @ApiOperation("Get all user list with pagination")
    @AuditLogs
    public ResponseEntity<?> getAllUsers( @RequestParam(defaultValue = "0") Integer pageNo, 
            @RequestParam(defaultValue = "10") Integer pageSize,
            @RequestParam(defaultValue = "id") String sortBy) {
    	Pageable paging = PageRequest.of(pageNo, pageSize, Sort.by(sortBy));
    	
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "User List", userService.findAll(paging).map(user -> new UserDto(user)).getContent()), HttpStatus.OK);
    }
 
    
     
    @PostMapping("/user")
    @ApiOperation("Create user")
    public ResponseEntity<?> createUser(@RequestBody UserDto user) {
    	User newUSer = null;
    	try {
	    	if(user.getId() == null || userService.findById(user.getId()) != null) {
	    		newUSer =  userService.save(user);
	    	}
    	}catch(ResourceNotFoundException e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "User Not found", user.getId()), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "User creation failed", user.getId()), HttpStatus.BAD_REQUEST);
    	}
    	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "User added successfully", new UserDto(newUSer)), HttpStatus.OK);
    	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }

    @PutMapping("/user")
    @ApiOperation("Update user")
    public ResponseEntity<?> updateUser(@RequestBody UserDto user) {
    	try {
    		User newUSer =  userService.save(user);
    		return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "User updated successfully", new UserDto(newUSer)), HttpStatus.OK);
    	}catch(DataIntegrityViolationException e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "User data already exist", null), HttpStatus.ALREADY_REPORTED);
    	}catch(ResourceNotFoundException e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.INVALID_USER, true, "User record not found", null), HttpStatus.NOT_FOUND);
    	}catch(Exception e) {
    		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "User updation failed", null), HttpStatus.BAD_REQUEST);
    	}
    	//return new ResponseEntity(new ApiResponse(StatusCode.DEPENDANCY_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
    }

}
