package com.ic.security;

import java.io.IOException;
import java.util.Date;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ic.constant.DBConfig;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.MalformedJwtException;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.SignatureException; 
import io.jsonwebtoken.UnsupportedJwtException;

@Component
public class JwtTokenProvider {

	private static final Logger LOG = LoggerFactory.getLogger(JwtTokenProvider.class);

	/*
	 * @Value("${app.jwtSecret}") private String jwtSecret;
	 * 
	 * @Value("${app.jwtExpirationInMs}") private int jwtExpirationInMs;
	 * 
	 * @Value("${secureKey}") private String key;
	 * 
	 * @Value("${secureInitVector}") private String initVector;
	 */
	
	@Autowired 
	private ObjectMapper objectMapper;
	
	@Autowired
	DBConfig DBConfig;

	public String generateToken(Authentication authentication) throws JsonProcessingException {

		UserPrincipal userPrincipal = (UserPrincipal) authentication.getPrincipal();

		Date now = new Date();
		Date expiryDate = new Date(now.getTime() + Long.parseLong(DBConfig.getValue("app.jwtExpirationInMs")));

		userPrincipal.setAuthorities(null);
		
		//JwtCustomPayload jwtCustomPayload = new JwtCustomPayload(userPrincipal.getUsername(), userPrincipal.getRoleType() , userPrincipal.getTenant());
		return EncrypterDecrypter.encrypt(DBConfig.getValue("secureKey"), DBConfig.getValue("secureInitVector"), Jwts.builder().setSubject(objectMapper.writeValueAsString(userPrincipal)).setIssuedAt(new Date()).setExpiration(expiryDate).signWith(SignatureAlgorithm.HS512, DBConfig.getValue("app.jwtSecret")).compact());
	}

	public String generateToken(UserPrincipal userPrincipal) throws JsonProcessingException {

		//UserPrincipal userPrincipal = (UserPrincipal) authentication.getPrincipal();

		Date now = new Date();
		Date expiryDate = new Date(now.getTime() + Long.parseLong(DBConfig.getValue("app.jwtExpirationInMs")));

		userPrincipal.setAuthorities(null);
		//JwtCustomPayload jwtCustomPayload = new JwtCustomPayload(userPrincipal.getUsername(), userPrincipal.getRoleType(), userPrincipal.getTenant());

		return EncrypterDecrypter.encrypt(DBConfig.getValue("secureKey"), DBConfig.getValue("secureInitVector"), Jwts.builder().setSubject(objectMapper.writeValueAsString(userPrincipal)).setIssuedAt(new Date()).setExpiration(expiryDate).signWith(SignatureAlgorithm.HS512, DBConfig.getValue("app.jwtSecret")).compact());
	}
	
	
	public UserPrincipal getJWT(String token) throws JsonParseException, JsonMappingException, IOException {
		Claims claims = Jwts.parser().setSigningKey(DBConfig.getValue("app.jwtSecret")).parseClaimsJws(token).getBody();
		UserPrincipal jwtCustomPayload = objectMapper.readValue(claims.getSubject(), UserPrincipal.class);
		//SMSContextHolder.setCountryCode(jwtCustomPayload.getCountryCode());
		LOG.info("Parse Token: {}.", jwtCustomPayload.getRole());
		return jwtCustomPayload;
	}

	public String getRoleTypeFromJWT(String token) throws JsonParseException, JsonMappingException, IOException {
		Claims claims = Jwts.parser().setSigningKey(DBConfig.getValue("app.jwtSecret")).parseClaimsJws(token).getBody();
		UserPrincipal jwtCustomPayload = objectMapper.readValue(claims.getSubject(), UserPrincipal.class);
		return jwtCustomPayload.getRole();
	}

	public boolean validateToken(String authToken) {
		try {
			Jwts.parser().setSigningKey(DBConfig.getValue("app.jwtSecret")).parseClaimsJws(authToken);
			return true;
		} catch (SignatureException ex) {
			LOG.error("Invalid JWT signature");
		} catch (MalformedJwtException ex) {
			LOG.error("Invalid JWT token");
		} catch (ExpiredJwtException ex) {
			LOG.error("Expired JWT token");
		} catch (UnsupportedJwtException ex) {
			LOG.error("Unsupported JWT token");
		} catch (IllegalArgumentException ex) {
			LOG.error("JWT claims string is empty.");
		}
		return false;
	}
}
